﻿/// <reference path="../Scripts/jquery.cookie.js" />
/// <reference path="../Scripts/modal.js" />

// constants
var getLanguage = false;

// Used by both:
// - widget's invisible iframe
// - widget's modal iframe
function postMessageToWidget(message) {
    var sent = false;
    
    if (window.parent && parent.postMessage) {
        $(PostMessageTargets).each(function (index, value) {
            parent.postMessage(JSON.stringify(message), value);
            sent = true;
        });
    }

    debugOutgoingMessage("InternalWidget.js.postMessageToWidget", message, sent, "widget");
}

// Used by:
// - widget's invisible iframe
// These are messages that the widget posts to the invisible iframe
function invisibleWidgetEventListener(event) {
    // All of the messages below come from the widget running in the host site's context.
    // So there's no reasonable way to check the origin.

    if (event && event.data) {
        var eventData = JSON.parse(event.data);

        if (eventData === "requestLogout") {
            debugIncomingMessage("InternalWidget.js.invisibleWidgetEventListener", event, true);
            logout();
            return false; // handled, so cancel any fallback behavior
        }
        else if (eventData === "getLanguage") {
            debugIncomingMessage("InternalWidget.js.invisibleWidgetEventListener", event, true);
            getLanguage = true;
            return false; // handled, so cancel any fallback behavior
        }
    }

    debugIncomingMessage("InternalWidget.js.invisibleWidgetEventListener", event, false);
    return true; // we didn't handle the event, so perform fallback behavior, if any
}

// Used by:
// - widget's modal iframe
// These are messages that the widget posts to the modal iframe
function modalWidgetEventListener(event) {
    // All of the messages below come from the widget running in the host site's context.
    // So there's no reasonable way to check the origin.

    var eventData = JSON.parse(event.data);

    if (event && event.data) {
        if (eventData && eventData.message && eventData.message === "youAreWidgetModal") {
            debugIncomingMessage("InternalWidget.js.modalWidgetEventListener", event, true);
            modalIsReady(true, eventData);

            return false; // handled, so cancel any fallback behavior
        }
        else if (eventData.length == 2 && eventData[0] === "contextDataReady" && eventData[1] != null) {
            debugIncomingMessage("InternalWidget.js.modalWidgetEventListener", event, true);
            $.event.trigger({
                type: "contextDataReady",
                message: eventData[1],
                time: new Date()
            });
            return false; // handled, so cancel any fallback behavior
        }
        else if (eventData && eventData.message && eventData.message === "setDeviceOrientation") {
            setOrientation(eventData);
            return false;
        }
    }

    debugIncomingMessage("InternalWidget.js.modalWidgetEventListener", event, false);
    return true; // we didn't handle the event, so perform fallback behavior, if any
}

// Used only by:
// - widget's invisible iframe
// Script view /Widget/InitInternalWidget
function initInternalWidget(checkLogin, requiredFieldSet) {
    debug("initInternalWidget(" + checkLogin + ") starting");
    
    if (window.addEventListener) {
        addEventListener("message", invisibleWidgetEventListener, false);
    } else {
        attachEvent("onmessage", invisibleWidgetEventListener);
    }

    // Check for presence of email verification token.
    var emailVerificationToken = $.cookie("EmailVerificationToken");

    if (checkLogin === true) {
        var samlWidget = getSamlWidget();
        if (samlWidget) {
            try {
                // If email verification token is defined, then log them out.
                if (emailVerificationToken) {
                    samlWidget.dropSession();
                } else {
                    samlWidget.loadconfiguration.loadpolicyvariables();
                    samlWidget.loadconfiguration.loadSite();

                    samlWidget.ESSOContext.saveToken = samlWidget.ESSOConstants.essoTrue;
                    if (samlWidget.loadconfiguration.load() == samlWidget.ESSOConstants.essoTrue) { // ESSO JS Client finds valid session
                        window.location.href = "/Widget/CheckLogin?requiredFieldSets=" + requiredFieldSet;
                        debug("initInternalWidget redirected to CheckLogin");
                        return false; // handled, so cancel any fallback behavior
                    }
                }
            } catch (e) {
                debug(e);
            }
        }

        internalWidgetIsReady("checkLoginCanceled");
    } else {
        internalWidgetIsReady("checkLoginSkipped");
    }

    debug("initInternalWidget ending");
    return false; // handled, so cancel any fallback behavior
}

function getSamlWidget() {
    var jqo = $("#samlWidget");
    if (jqo) {
        var elem = jqo.get(0);
        if (elem) {
            var contentWindow = elem.contentWindow;
            if (contentWindow) {
                return contentWindow;
            }
        }
    }
    return null;
}

function internalWidgetIsReady(checkLoginCase) {
    var samlToken = null;
    var rememberMeToken = null;
    var emailVerificationToken = $.cookie("EmailVerificationToken");
    if (emailVerificationToken) {
        $.removeCookie("EmailVerificationToken", { path: '/', domain: AccountsCookieDomain });
    }

    if (checkLoginCase === "checkLoginCompleted") {
        var success = false;
        
        var samlWidget = getSamlWidget();
        if (samlWidget) {
            try {
                samlWidget.loadconfiguration.loadpolicyvariables();
                samlWidget.loadconfiguration.loadSite();
                if (samlWidget.loadconfiguration.load() == samlWidget.ESSOConstants.essoTrue) { // ESSO JS Client finds valid session - should always be true at this point, but just in case....
                    samlToken = samlWidget.ESSOContext.samlToken.SAMLTokenValue;
                    rememberMeToken = samlWidget.ESSOContext.rememberMeToken;
                    success = true;
                }
            } catch (e) {
                debug(e);
            }
        }
        
        if (!success) {
            // recurse
            internalWidgetIsReady("checkLoginCanceled");
            return;
        }
    }

    var message = {
        message: "internalWidgetReady",
        checkLoginCase: checkLoginCase,
        sessionToken: samlToken,
        rememberMeToken: rememberMeToken,
        isMobile: window.IsMobile, 
        languageCode: InternalWidgetConfig.languageCode,
        buttonText: InternalWidgetConfig.buttonText,
        emailVerificationToken: emailVerificationToken
    };
    
    postMessageToWidget(message);
}

// Used only by:
// - widget's invisible iframe
// Called by widgetEventListener when it in turn is called by widget.
function logout() {
    debug("logout starting");
    
    try {
        getSamlWidget().dropSession();
    } catch(e) {
        debug(e);
    }
    
    removeAllCookies();
    postMessageToWidget("loggedOut");

    debug("logout ending");
    return false; // handled, so cancel any fallback behavior
}

// Used only by:
// - widget's modal iframe
// Called by any modal that wants to reload state after detour to set language
function readyForContextData() {
    debug("readyForContextData starting");
    
    postMessageToWidget("readyForContextData");

    debug("readyForContextData ending");
}

// Used only by:
// - widget's modal iframe
// Called by any modal that wants to reload state after detour to set language
function cancelLogin() {
    debug("cancelLogin starting");
    
    try {
        getSamlWidget().dropSession();
    } catch(e) {
        debug(e);
    }
    
    removeAllCookies();
    postMessageToWidget("canceled");

    debug("cancelLogin ending");
    return false; // handled, so cancel any fallback behavior
}

// Used only by:
// - widget's modal iframe
// Called by script-only view CompleteLogin, which is usually the last view of successful login sequence.
function handleLoggedIn(overrideUrl) {
    debug("handleLoggedIn starting");

    var samlWidget = getSamlWidget();
    if (samlWidget) {
        try {

            samlWidget.loadconfiguration.loadpolicyvariables();
            samlWidget.loadconfiguration.loadSite();

            var sessionTokenString = $.cookie("loginSessionToken");
            var sessionTokenObj = samlWidget.jQuery.deparam(sessionTokenString);
            var rememberMeTokenString = $.cookie("loginRememberMeToken");
            var rememberMeTokenObj = samlWidget.jQuery.deparam(rememberMeTokenString);

            $.removeCookie("loginSessionToken", { path: '/', domain: AccountsCookieDomain });
            $.removeCookie("loginRememberMeToken", { path: '/', domain: AccountsCookieDomain });
            $.removeCookie("WidgetFollowUps", { path: '/', domain: AccountsCookieDomain });

            samlWidget.saveTokensInCookies(sessionTokenObj, rememberMeTokenObj.RememberMeToken);


            if (!!overrideUrl) {
                window.location.href = overrideUrl;
            } else {
                postLoggedInMessageToWidget();
            }

            debug("handleLoggedIn ending");
            return false; // handled, so cancel any fallback behavior
        } catch (e) {
            debug(e);
        }
    }

    // The above failed for who knows what reason, so cancel login.
    cancelLogin();
    
    debug("handleLoggedIn ending");
    return false; // handled, so cancel any fallback behavior
}

function postLoggedInMessageToWidget() {

    var samlWidget = getSamlWidget();
    var samlToken = samlWidget.authenticateEntity.getSAMLToken().SAMLTokenValue;
    var rememberMeToken = samlWidget.authenticateEntity.getRememberMeToken();
    var data = ["loggedIn", samlToken, rememberMeToken];
    postMessageToWidget(data);
}

// Probably merge this into the initialization stuff and get rid of this.
function handleLanguageCheckComplete(language) {
    debug("handleLanguageCheckComplete starting");

    var data = ["languageCheck", language];
    postMessageToWidget(data);

    debug("handleLanguageCheckComplete ending");
}

function handleLanguageChanged(reloadPageOnClose, openPreviousModalOnClose, contextData, previousModalUrl, languageActuallyChanged, languageSelection) {
    debug("handleLanguageChanged starting");

    var data = ["languageChanged", reloadPageOnClose, openPreviousModalOnClose, contextData, previousModalUrl, languageActuallyChanged, languageSelection];
    postMessageToWidget(data);

    debug("handleLanguageChanged ending");
    return false; // handled, so cancel any fallback behavior
}

function login(username, password, rememberMeFlag, onLoginError, requiredFieldSets) {
    
    debug("login starting with " + stringify("arguments", {
        username: username,
        rememberMeFlag: rememberMeFlag,
        requiredFieldSets: requiredFieldSets
    }));

    var samlWidget = getSamlWidget();
    
    if (samlWidget) {
        try {
            samlWidget.dropSession();

            if (samlWidget.authenticate(username, password, rememberMeFlag, false) == samlWidget.ESSOConstants.essoTrue) {
                // OK, login and password check out, but we still need to:
                // - check version (migrate legacy)
                // - check t&c (get new terms signed)
                // - check required fields (get required fields)
                // - check for additional actions (e.g., claim win)
                // We do all of the above like this:
                // - Save the session token & remember me tokens to special cookies.
                // - Navigate (GET) to controller action CheckLoginComplete action.
                //   This action checks all of the above in sequence.
                // - If it hits anything that needs doing, then it returns the modal form that does that operation.
                //   In that modal form, it reproduces the hidden inputs for session and remember me tokens.
                //   That form posts to an action URL to receive the specific post for that operation.
                //   If successful, then that action redirects back to the CheckLoginComplete action and we loop.
                //   If unsuccessful, then that action redraws its form for the user to try again.
                // - If it doesn't hit anything that needs doing, then CheckLoginComplete returns the CompleteLogin view.
                //   This view renders script which calls InternalWidget's handleLoggedIn method.

                var sessionTokenObj = samlWidget.authenticateEntity.getSAMLToken();
                if (sessionTokenObj) {
                    var sessionTokenString = $.param(sessionTokenObj);
                    var rememberMeTokenObj = {
                        RememberMeToken: samlWidget.authenticateEntity.getRememberMeToken()
                    };
                    
                    var rememberMeTokenString = $.param(rememberMeTokenObj);

                    $.cookie("loginSessionToken", sessionTokenString, { path: '/', domain: AccountsCookieDomain });
                    $.cookie("loginRememberMeToken", rememberMeTokenString, { path: '/', domain: AccountsCookieDomain });

                    window.location.href = "/Widget/CompleteLogin?requiredFieldSets=" + requiredFieldSets;

                    debug("login ending");
                    return false; // handled, so cancel any fallback behavior
                }
            }
        } catch (e) {
            debug(e);
        }
    }

    onLoginError();

    debug("login ending");
    return false; // handled, so cancel any fallback behavior
}

// Object for passing around the language modal options in
// events.
function languageModalOptions() {
    this.ReloadPageOnClose = false;
    this.OpenPreviousModalOnClose = false;
    this.PreviousModalUrl = null;
    this.ContextData = new Object();
}

// Object for passing around a flag for whether
// the modal closed event was handled or not in 
// events.
function modalClosedOptions() {
    this.handled = false;
}
