// ReSharper disable InconsistentNaming
var wotcAccountWidget = new WotcAccountWidget();

function WotcAccountWidget() {
    // private functions
    var _getMaintenanceModalUrl = function() {
        return _accountsBaseUrl + _maintenanceModalPath;
    };
    
    var _getIsSiteUpUrl = function () {
        return _accountsBaseUrl + _isSiteUpPath;
    };

    var _getAccountDetailsPageUrl = function () {
        return _accountsBaseUrl + _accountDetailsPagePath;
    };
    
    var _getSupportStylesheetUrl = function() {
        return _accountsBaseUrl + _supportStylesheetPath;
    }

    var _getLanguageModalUrl = function () {
        return _accountsBaseUrl + _languageModalPath;
    };

    var _getLoginModalUrl = function (showCreateButton) {
        var url = _accountsBaseUrl + _loginModalPath + "?returnUrl=" + encodeURIComponent(window.location.href) + "&requiredFieldSets=" + _requiredFieldSets;
        if (showCreateButton === true) {
            //Query string value is not signification here, if parameter 'showCreateButton' exist then we'll show create button
            url += "&showCreateButton=true";
        }
        return url;
    };

    var _getJoinModalUrl = function () {
        return _accountsBaseUrl + _joinModalPath + "?returnUrl=" + encodeURIComponent(window.location.href) + "&requiredFieldSets=" + _requiredFieldSets;
    };

    var _getVerifyEmailModalUrl = function (emailVerificationToken) {
        return _accountsBaseUrl + _verifyEmailModalPath + "/" + emailVerificationToken + "?requiredFieldSets=" + _requiredFieldSets;
    };

    var _getInitInternalWidgetUrl = function (checkLogin, requiredFieldSets) {

        // NOTE: This is vestigal code. The internal widget controller used
        // to set required field sets in the cookie. Since that is now based
        // on query string, the code no longer does anything. However, it is
        // safer to leave it in place at the moment until we have time to
        // properly excise it.
        if (requiredFieldSets && requiredFieldSets.length && requiredFieldSets.length > 0) {
            requiredFieldSets = encodeURIComponent(requiredFieldSets.replace(/\+/g, ","));
        } else {
            requiredFieldSets = "";
        }
        var url = _accountsBaseUrl +
            (checkLogin ? _initInternalWidgetWithCheckLoginPath : _initInternalWidgetPath) +
            "/" + requiredFieldSets;

        return url;
    };

    var _createElement = function (tagName, attributes) {
        var result = document.createElement(tagName);

        if (attributes) {
            for (var key in attributes) {
                result.setAttribute(key, attributes[key]);
            }
        }
        return result;
    };

    var _getRoot = function () {
        return document.documentElement;
    };

    var _getModalIframeElement = function () {
        return document.getElementById(_modalIframeName);
    };

    var _getInternalWidgetIframeElement = function () {
        return document.getElementById(_internalIframeName);
    };

    var _showModal = function (source) {
        _debug("_showModal(" + source + ") starting");
        
        if (!_isSiteUp) {
            _debug("_showModal(" + source + "): Site is down. Showing maintenance message.");
            source = _getMaintenanceModalUrl();
        }

        // TODO: Figure out where this logic needs to be. I don't think the Widget is
        // cleaning up its iframes, either that or I'm not using the API properly? This is kind
        // of important for the messaging to go to the "intended" place.
        var oldModalIframe = _getModalIframeElement();
        if (oldModalIframe) {
            oldModalIframe.parentNode.removeChild(oldModalIframe);
        }

        var modalIframe = _createElement('iframe', { id: _modalIframeName, src: source });

        _buildUpModal(modalIframe);

        _debug("_showModal(" + source + ") ending");
        return false; // handled, so cancel any fallback behavior
    };

    var _freezePage = function (modalIframe) {

        _addClass(modalIframe, "wotcWidget-modalIFrame");
               
        modalIframe.style.zIndex = _getTopZIndex();
        
        var body = document.getElementsByTagName("body")[0];

        if (_isMobile) {
            if (0 <= navigator.userAgent.indexOf("iPhone")) {
                _addClass(modalIframe, "wotcWidget-iphone");
            }
            
            _replaceParentPageBodyWithNewContent(modalIframe);
            _temporarilySetViewportScale("width=device-width, initial-scale=1.0");
        } else {
            _addClass(body, "wotcWidget-modalDisplayed");
            
            if (0 <= navigator.userAgent.indexOf("Nexus")) {
                _addClass(body, "nexusTablet");
            }

            if (navigator.userAgent.indexOf("iPad") >= 0) {
                _addClass(modalIframe, "wotcWidget-ipad");
            
                var touchScrollWrapper = document.createElement("div");
                
                touchScrollWrapper.id = _touchScrollWrapperName;
                
                _addClass(touchScrollWrapper, "wotcWidget-touchScrollWrapper");
                
                touchScrollWrapper.appendChild(modalIframe);
                
                touchScrollWrapper.style.zIndex = modalIframe.style.zIndex;

                body.appendChild(touchScrollWrapper);
            } else {
                body.appendChild(modalIframe);
            }
        }

        return false; // handled, so cancel any fallback behavior
    };

    var _temporarilySetViewportScale = function(content) {
        var metaTags = document.getElementsByTagName("meta");
        var viewPortTag = null;

        for (var i = 0; i < metaTags.length; i++) {
            var node = metaTags[i];
            if (node.name == "viewport") {
                viewPortTag = node;
                break;
            }
        }

        if (viewPortTag == null) {
            viewPortTag = document.createElement('meta');
            viewPortTag.id = "viewport";
            viewPortTag.name = "viewport";
            viewPortTag.addedByWidget = true;

            if (document.getElementsByTagName("head").length > 0) {
                document.getElementsByTagName("head")[0].appendChild(viewPortTag);
            }
        }

        viewPortTag.content_old = viewPortTag.content;
        viewPortTag.content = content;
    };

    var _revertViewportScale = function() {
        var metaTags = document.getElementsByTagName("meta");

        for (var i = 0; i < metaTags.length; i++) {
            var node = metaTags[i];
            if (node.name == "viewport") {
                if (node.addedByWidget) {
                    node.parentNode.removeChild(node);
                } else {
                    node.content = node.content_old;
                    node.content_old = null;
                }

                break;
            }
        }
    };

    var _replaceParentPageBodyWithNewContent = function(newContent) {
        // This approach 'hides' the main page content, but does not disable it.
        // For example, if there's a form or event or something that's happening in the main page
        // DOM it won't stop. It also won't prevent widget features, like reopening the previous modal
        // after language selection from working. That relies on a form in the main page to persist data
        // across requests, if the form was removed from the DOM it would stop working.
        var container = document.createElement("div");
        _addClass(container, "wotcWidget-offscreen");
        document.body.appendChild(container);

        var nodes = new Array();

        for (var i = 0; i < document.body.childNodes.length; i++) {
            var node = document.body.childNodes[i];

            if (node != container) {
                nodes.push(node);
            }
        }

        while (nodes.length > 0) {
            container.appendChild(nodes.shift());
        }

        _parentPageBody = container;
        _addClass(document.body, "wotcWidget-replacedContent");
        _addClass(document.documentElement, "wotcWidget-replacedContent");

        document.body.appendChild(newContent);
    };

    var _revertParentPageBodyToNormal = function() {
        var nodes = new Array();

        for (var i = 0; i < _parentPageBody.childNodes.length; i++) {
            nodes.push(_parentPageBody.childNodes[i]);
        }

        _revertViewportScale();
        _removeClass(document.body, "wotcWidget-replacedContent");
        _removeClass(document.documentElement, "wotcWidget-replacedContent");

        while (nodes.length > 0) {
            document.body.appendChild(nodes.shift());
        }

        _parentPageBody.parentNode.removeChild(_parentPageBody);
        _parentPageBody = null;
    };

    var _hasClass = function (element, className) {
        if (null == element || null == className || "" == className) {
            return false;
        }

        if (-1 < element.className.indexOf(className)) {
            return true;
        } else {
            return false;
        }
    };

    var _addClass = function (element, className) {
        if (null == element || null == className || "" == className) {
            return;
        }

        if (!_hasClass(element, className)) {
            element.className += " " + className;
        }
    };

    var _removeClass = function (element, className) {
        if (null == element || null == className || "" == className) {
            return;
        }

        element.className = element.className.replace(
                new RegExp('(^|\\s+)' + className + '(\\s+|$)', 'g'), '');
    };

    var _buildUpModal = function (modalIframe) {
        _freezePage(modalIframe);
        return false; // handled, so cancel any fallback behavior
    };

    var _tearDownModal = function () {
        _debug("_tearDownModal starting");

        var modalIframe = _getModalIframeElement();
        modalIframe.parentNode.removeChild(modalIframe);
        
        var touchScrollWrapper = document.getElementById(_touchScrollWrapperName);
        if (touchScrollWrapper != null) {
            touchScrollWrapper.parentNode.removeChild(touchScrollWrapper);
        }

        if (_isMobile) {
            _revertParentPageBodyToNormal();
        } else {
            var body = document.getElementsByTagName("body")[0];
            _removeClass(body, "wotcWidget-modalDisplayed");
            _removeClass(body, "nexusTablet");
            _removeClass(body, "wotcWidget-ipad");
        }
        _debug("_tearDownModal ending");
        return modalIframe;
    };

    var _addWidgetEventListener = function () {
        if (window.addEventListener) {
            addEventListener("message", _widgetEventListener, false);
        } else {
            attachEvent("onmessage", _widgetEventListener);
        }
    };

    var _safeBool = function (val) {
        if ("string" == typeof (val)) {
            val = val.toLowerCase();
        }

        if (val === true) {
            return true;
        }

        if (val === "true") {
            return true;
        }

        return false;
    };

    var _safeFunction = function (val) {
        if (val &&
            typeof val == "function") {
            return val;
        }
        return null;
    };

    var _handleLoggedIn = function (sessionToken, rememberMeToken) {
        try {
            _debug("_handleLoggedIn starting");

            _isLoggedIn = true;

            if (_onLoggedIn) {
                // Host page gave us a callback function. Call it and evaluate the result.
                if (!_onLoggedIn(sessionToken, rememberMeToken, _inRequireAuthentication)) {
                    // Host page is telling us they've fully handled the event
                    // (probably by navigating or reloading the page)
                    // and they don't want us to perform the fallback behavior.
                    return false; // handled, so cancel any fallback behavior
                }
            }

            // If we're in the "requireAuthentication" sequence, then the _onLoggedIn callback above still has a chance to say it's fully
            // handled the event. Hance the return false above. If it does *not* return false, then we fall through to here.
            // The default behavior for the requireAuthentication sequence is to call the _requireAuthenticationCallback method.
            // If we're not in the requireAuthentication sequence, then the default behavior is to reload the page.
            if (_inRequireAuthentication) {
                if (_requireAuthenticationCallback) {
                    _requireAuthenticationCallback();
                }
            } else {
                // Host page did not give us a callback. Do the default thing, which is to reload the page so that it can pick up the cookie and load itself in logged-in state.
                window.location.reload(true);
            }
            return false; // handled, so cancel any fallback behavior
        } catch (e) {
            _debug(e);
        } finally {
            _cleanUpRequireAuthenticationSequence();
            _debug("_handleLoggedIn ending");
        }
    };

    var _handleLoginCanceled = function () {
        try {
            _debug("_handleLoginCanceled starting");

            if (_onLoginCanceled) {
                // Host page gave us a callback. Call it and evaluate the result.
                if (!_onLoginCanceled()) {
                    // Host page is telling us they've fully handled the event
                    // (Who knows how? Navigate? Reload?)
                    // and they don't want us to perform the fallback behavior.
                    return false; // handled, so cancel any fallback behavior
                }
            }

            // Either:
            // - Host page did not give us a callback. -
            // - or -
            // - Host page callback is telling us they've not fully handled the event, so we should do the fallback behavior. -
            // Either way, do the default thing, which is to tear down the modal.
            _hideModal();
            return false; // handled, so cancel any fallback behavior
        } catch (e) {
            _debug(e);
        } finally {
            _cleanUpRequireAuthenticationSequence();
            _debug("_handleLoginCanceled ending");
        }
    };

    var _handleLoggedOut = function () {
        try {
            _debug("_handleLoggedOut starting");

            _isLoggedIn = false;

            if (_onLoggedOut) {
                // Host page gave us a callback. Call it and evaluate the result.
                if (!_onLoggedOut()) {
                    // Host page is telling us they've fully handled the event
                    // (Who knows how? Navigate? Reload?)
                    // and they don't want us to perform the fallback behavior.
                    return false; // handled, so cancel any fallback behavior
                }
            }

            // Either:
            // - Host page did not give us a callback. -
            // - or -
            // - Host page callback is telling us they've not fully handled the event, so we should do the fallback behavior. -
            // Either way, do the default thing, which is to reload the page so that it can load itself in logged-out state.
            window.location.reload(true);
            return false; // handled, so cancel any fallback behavior
        } catch (e) {
            debug(e);
        } finally {
            _cleanUpRequireAuthenticationSequence();
            _debug("_handleLoggedOut ending");
        }
    };

    var _handleLanguageChanged = function (reloadPageOnClose, openPreviousModalOnClose, contextData, previousModalUrl, languageActuallyChanged, languageSelection) {
        try {
            _debug("_handleLanguageChanged starting");
            reloadPageOnClose = _safeBool(reloadPageOnClose);
            openPreviousModalOnClose = _safeBool(openPreviousModalOnClose);
            languageActuallyChanged = _safeBool(languageActuallyChanged);

            _language = languageSelection;

            if (_onLanguageChanged) {
                // Host page gave us a callback. Call it and evaluate the result.
                if (!_onLanguageChanged(reloadPageOnClose, openPreviousModalOnClose, contextData, previousModalUrl, languageActuallyChanged, languageSelection)) {
                    // Host page is telling us they've fully handled the event
                    // (Who knows how? Navigate? Reload?)
                    // and they don't want us to perform the fallback behavior.
                    return false; // handled, so cancel any fallback behavior
                }
            }

            // Either:
            // - Host page did not give us a callback. -
            // - or -
            // - Host page callback is telling us they've not fully handled the event, so we should do the fallback behavior. -
            // Either way, do the default thing, which is to close the modal or reopen the last modal url. You would think default would be to reload the page
            // but that is not the case because we don't know if the page is a) localized and b) uses our language setting.
            if (openPreviousModalOnClose && previousModalUrl && previousModalUrl != '') {
                _contextData = contextData; // Set the context data so it's available for modals that want it.
                _showModal(_accountsBaseUrl + previousModalUrl); // This should work because both Join and Login set their action URL.
            } else {
                _hideModal();
            }

            return false; // handled, so cancel any fallback behavior
        } catch (e) {
            debug(e);
        } finally {
            _debug("_handleLanguageChanged ending");
        }
    };

    var _widgetEventListener = function (event) {
        if (event && event.data && event.origin === _accountsBaseUrl) {

            var eventData = JSON.parse(event.data);
            
            var message = eventData;
            if (eventData.message) {
                message = eventData.message;
            }

            if (message === "siteisup") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                _isSiteUp = true;
                return false;
            } else if (eventData && eventData.message && eventData.message === "internalWidgetReady") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                _initComplete(eventData);
                return false; // handled, so cancel any fallback behavior
            } else if (_isArray(eventData) && eventData.length && eventData.length == 3 && eventData[0] === "loggedIn") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                var sessionToken = eventData[1];
                var rememberMeToken = eventData[2];
                return _handleLoggedIn(sessionToken, rememberMeToken);
            } else if (_isArray(eventData) && eventData.length && eventData.length == 2 && eventData[0] === "languageCheck") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                _language = eventData[1];
                return false;
            } else if (_isArray(eventData) && eventData.length && eventData.length == 7 && eventData[0] === "languageChanged") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                var reloadPageOnClose = eventData[1];
                var openPreviousModalOnClose = eventData[2];
                var contextData = eventData[3];
                var previousModalUrl = eventData[4];
                var languageActuallyChanged = eventData[5];
                var languageSelection = eventData[6];

                return _handleLanguageChanged(reloadPageOnClose, openPreviousModalOnClose, contextData, previousModalUrl, languageActuallyChanged, languageSelection);
            } else if (eventData === "readyForContextData" && _contextData != null) {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                _postMessageToWidgetModal(["contextDataReady", _contextData]);
                _contextData = null; // The context data should only be used once.
                return false;
            } else if (eventData === "canceled") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                return _handleLoginCanceled();
            } else if (eventData === "loggedOut") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);
                _handleLoggedOut();
                return false; // handled, so cancel any fallback behavior
            } else if (eventData == "hideModal") {
                var iframes = document.getElementsByTagName("iframe");
                var iframeCount = iframes.length;
                for (var i = 0; i < iframeCount; ++i) {
                    if (_hasClass(iframes[i], "loaded")) {
                        _removeClass(iframes[i], "loaded");
                    }
                }
                
                return false;
            } else if (eventData === "modalReady") {
                _debugIncomingMessage("widget.js._widgetEventListener", event, true);

                var data = {
                    message: "youAreWidgetModal",
                    isMobile: _isMobile
                };

                _postMessageToWidgetModal(data);

                // TODO: If any widget level refactoring occurs, this needs to be refactored too.
                // I need the orientation info from the parent window, 
                // the way we include the modal.js causes modalReady is sent to widget.js for non widget modals as well
                // not sure how we are going to refactor this, so I just utilize the current infrastructure for now.

                data.message = "youAreNonWidgetModal";

                if (typeof postMessageToNonWidgetModal == 'function') {
                    postMessageToNonWidgetModal(data);
                }

                return false;
            } else if (eventData && eventData === "refreshDciCardForMobile") { // modal -> parent
                refreshDciCard(_reserveUpdatedDciImageUrlForMobile);
                return false;
            } 
        }

        _debugIncomingMessage("widget.js._widgetEventListener", event, false);
        return true; // we didn't handle the event, so perform fallback behavior, if any
    };

    var _getTopZIndex = function () {
        var topZ = 0;

        var elements = document.getElementsByTagName("*");

        for (var i = 0; i < elements.length; i++) {
            var item = elements[i];
            var computedZIndex = document.defaultView.getComputedStyle(item, null).getPropertyValue("z-index");
            if (computedZIndex) {
                var z = parseInt(computedZIndex);
                if (z > topZ && item.id !== "nonWidgetModalIframe") {
                    topZ = z;
                }
            }
        }

        return topZ + 1;
    };

    var _buildUpInternalIframe = function (source) {
        var iframe = _createElement("iframe", { id: _internalIframeName, src: source, style: "display:none;" });

        document.getElementsByTagName("body")[0].appendChild(iframe);

        return iframe;
    };

    var _showLoginModal = function (showCreateButton) {
        return _showModal(_getLoginModalUrl(showCreateButton));
    };

    var _showJoinModal = function () {
        return _showModal(_getJoinModalUrl());
    };

    var _showAccountDetails = function (newTab) {
        var url = _getAccountDetailsPageUrl();
        if (!newTab) {
            window.location.href = url;
        } else {
            window.open(url);
        }
        return false;
    };

    var _showLanguageModal = function (reloadPageOnClose, openPreviousModalOnClose, previousModalUrl, contextData) {
        reloadPageOnClose = _safeBool(reloadPageOnClose);
        openPreviousModalOnClose = _safeBool(openPreviousModalOnClose);

        var modalUrl = _getLanguageModalUrl() + '?';
        if (reloadPageOnClose != null) {
            modalUrl += '&reloadPageOnClose=' + reloadPageOnClose;
        }
        if (openPreviousModalOnClose != null) {
            modalUrl += '&openPreviousModalOnClose=' + openPreviousModalOnClose;
        }
        if (previousModalUrl != null) {
            modalUrl += '&previousModalUrl=' + previousModalUrl;
        }
        return _showModal(modalUrl);
    };

    var _showVerifyEmailModal = function (emailVerificationToken) {
        return _showModal(_getVerifyEmailModalUrl(emailVerificationToken));
    };

    var _hideModal = function () {
        _tearDownModal();

        return false; // handled, so cancel any fallback behavior
    };

    var _postMessageToWidgetModal = function (message) {
        var sent = false;
        var iframe = _getModalIframeElement();
        if (iframe) {
            if (iframe && iframe.contentWindow && iframe.contentWindow.postMessage) {
                iframe.contentWindow.postMessage(JSON.stringify(message), _accountsBaseUrl);
                sent = true;
            }
        }
        _debugOutgoingMessage("Widget.js._postMessageToWidgetModal", message, sent, "widget modal");
        return !sent; // If sent, return false to cancel fallback behavior. If not sent, return true to run fallback behavior.
    };

    var _postMessageToInternalWidget = function (message) {
        var sent = false;
        var iframe = _getInternalWidgetIframeElement();
        if (iframe) {
            if (iframe && iframe.contentWindow && iframe.contentWindow.postMessage) {
                iframe.contentWindow.postMessage(JSON.stringify(message), _accountsBaseUrl);
                sent = true;
            }
        }
        _debugOutgoingMessage("Widget.js._postMessageToInternalWidget", message, sent, "internal widget");
        return !sent; // If sent, return false to cancel fallback behavior. If not sent, return true to run fallback behavior.
    };

    var _logOut = function () {
        if ("undefined" != typeof confirmChanges && null != confirmChanges && true === confirmChanges) {
            if ("undefined" == typeof _buttonText.confirmLogOutMessage || null == _buttonText.confirmLogOutMessage) {
                _buttonText.confirmLogOutMessage = "You modified one or more fields on the current page. if you log out now the unsaved changes will be lost, continue?";
            }

            var confirmedLogOut = confirm(_buttonText.confirmLogOutMessage);
            if (true === confirmedLogOut) {
                _postMessageToInternalWidget("requestLogout");
                confirmChanges = false;
            }
        } else {
            _postMessageToInternalWidget("requestLogout");
        }
        
        return false; // handled, so cancel any fallback behavior
    };

    var _hackGetBaseUrlFromScriptTag = function () {
        var scripts = document.getElementsByTagName("script");
        var script = scripts[scripts.length - 1];
        var url = script.src;
        var pos = url.indexOf("/", 8);
        var base = url.substring(0, pos);
        return base;
    };

    var _isObject = function (o) {
        return o && typeof o === "object";
    };

    var _isArray = function (o) {
        return Array.isArray(o);
    };

    var _stringify = function (name, value) {
        var ret = "";
        if (name) {
            ret += name + ":";
        }
        if (_isObject(value)) {
            ret += "{";
            var hasProperties = false;
            Object.getOwnPropertyNames(value).forEach(function (propertyName, index, array) {
                if (hasProperties) {
                    ret += ",";
                } else {
                    hasProperties = true;
                }
                ret += _stringify(propertyName, value[propertyName]);
            });
            ret += "}";
        } else if (_isArray(value)) {
            ret += "[";
            var hasElements = false;
            value.forEach(function (element, index, array) {
                if (hasElements) {
                    ret += ",";
                } else {
                    hasElements = true;
                }
                ret += _stringify(null, element);
            });
            ret += "]";
        } else {
            ret += value;
        }
        return ret;
    };

    var _debug = function (text) {
        try {
            if (console && console.log) {
                console.log(text);
            }
        } catch (e) {
        }
    };

    var _debugIncomingMessage = function (handler, event, handled) {
        try {
            var message = ">"
                + (handled ? ">" : "|")
                + " ";
            if (event) {
                try {
                    var eventData = JSON.parse(event.data);
                    message += _stringify(null, eventData);
                } catch (e) {
                    message += event.data ? event.data : event;
                }
                if (event.origin) {
                    message += " origin:" + event.origin;
                }
            } else {
                message += event;
            }
            message += " receiver:"
                + handler;
            _debug(message);
        } catch (e) {
            _debug("error in _debugIncomingMessage: " + e);
        }
    };

    var _debugOutgoingMessage = function (sender, message, sent, target) {
        try {
            var text = (sent ? "<" : "|")
                + "< "
                + _stringify(null, message)
                + " sender:"
                + sender
                + " target:"
                + target;
            _debug(text);
        } catch (e) {
            _debug("error in _debugOutgoingMessage: " + e);
        }
    };

    var _addRequiredFieldSet = function (requiredFieldSet) {
        if (requiredFieldSet &&
            //requiredFieldSet.trim &&
            requiredFieldSet.toLowerCase) {
            requiredFieldSet = requiredFieldSet
                .trim()
                .toLowerCase();
            if (requiredFieldSet === _rfsName ||
                requiredFieldSet === _rfsPlayLocation ||
                requiredFieldSet === _rfsScreenName) {
                if (_requiredFieldSets !== "")
                    _requiredFieldSets += "+";
                _requiredFieldSets += requiredFieldSet;
            }
        }
    };

    var _setRequiredFieldSets = function (requiredFieldSets) {
        _requiredFieldSets = "";
        if (requiredFieldSets) {
            if (_isArray(requiredFieldSets)) {
                for (var i = 0; i < requiredFieldSets.length; i++) {
                    _addRequiredFieldSet(requiredFieldSets[i]);
                }
            } else if (requiredFieldSets.indexOf && (requiredFieldSets.indexOf("+") >= 0) && requiredFieldSets.split) {
                var arr = requiredFieldSets.split("+");
                _setRequiredFieldSets(arr);
            } else {
                _addRequiredFieldSet(requiredFieldSets);
            }
        }

        if (_requiredFieldSets == "") {
            // Don't remove this. See comment on ServiceWrapper.IsMissingRequiredFieldsInSet() for why.
            _requiredFieldSets = "default";
        }
    };

    var _cleanUpRequireAuthenticationSequence = function () {
        _inRequireAuthentication = false;
        _requireAuthenticationCallback = null;
    };

    var _requireAuthentication = function (requireAuthenticationCallback, defaultUsername) {
        _requireAuthenticationCallback = _safeFunction(requireAuthenticationCallback);
        _inRequireAuthentication = true;

        if (_isLoggedIn) {
            if (_requireAuthenticationCallback) {
                _requireAuthenticationCallback();
            }
            _cleanUpRequireAuthenticationSequence();
        } else {
            _showLoginModal(defaultUsername);
        }
    };

    var _getParameterByName = function (name) {
        name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
        var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
            results = regex.exec(location.search);
        return results == null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
    };

    var _getLoggedInControls = function (showMyAccount, showLogOut) {
        if (!showMyAccount && !showLogOut) {
            return null;
        }

        var container = _createElement("div", { "class": "wotc-account-widget-control-container" });
        var link;
        var span;

        if (showMyAccount) {
            link = _createElement("a", { href: _getAccountDetailsPageUrl(), target: "_blank", "class": "wotc-account-widget-button wotc-account-widget-button-myaccount" });
            span = _createElement("span");
            span.innerHTML = _buttonText.myAccount;
            link.appendChild(span);
            container.appendChild(link);
        }

        if (showLogOut) {
            link = _createElement("a", { href: "javascript:void(0);", target: "_blank", "class": "wotc-account-widget-button wotc-account-widget-button-signout", onclick: "return wotcAccountWidget.logOut();" });
            span = _createElement("span");
            span.innerHTML = _buttonText.logOut;
            link.appendChild(span);
            container.appendChild(link);
        }

        return container;
    };

    var _getLoggedOutControls = function (showJoin, showLogin) {
        if (!showJoin && !showLogin) {
            return null;
        }

        var container = _createElement("div", { "class": "wotc - account - widget - control - container" });
        var link;
        var span;

        if (showJoin) {
            link = _createElement("a", { href: "javascript:void(0);", "class": "wotc-account-widget-button wotc-account-widget-button-join", onclick: "return wotcAccountWidget.showJoinModal();" });
            span = _createElement("span");
            span.innerHTML = _buttonText.join;
            link.appendChild(span);
            container.appendChild(link);
        }

        if (showLogin) {
            link = _createElement("a", { href: "javascript:void(0);", "class": "wotc-account-widget-button wotc-account-widget-button-signin", onclick: "return wotcAccountWidget.showLoginModal();" });
            span = _createElement("span");
            span.innerHTML = _buttonText.login;
            link.appendChild(span);
            container.appendChild(link);
        }

        return container;
    };

    var _initStart = function (isLoggedIn, requiredFieldSets, container, onLoggedIn, onLoginCanceled, onLoggedOut, onLanguageChanged, hideMyAccount, hideLogOut, hideJoin, hideLogin, showLoginModal, showJoinModal, contextData) {
        if (_initStarted) {
            _debug("_initStart called (ignored; NOT the first time)");
            return;
        }
        _debug("_initStart starting (first time)");
        _initStarted = true;

        _sendIsSiteUpPing();

        _isLoggedIn = _safeBool(isLoggedIn);
        var checkLogin = !_isLoggedIn;

        _setRequiredFieldSets(requiredFieldSets);

        _container = container;

        _onLoggedIn = _safeFunction(onLoggedIn);
        _onLoginCanceled = _safeFunction(onLoginCanceled);
        _onLoggedOut = _safeFunction(onLoggedOut);
        _onLanguageChanged = _safeFunction(onLanguageChanged);

        _showMyAccount = !_safeBool(hideMyAccount);
        _showLogOut = !_safeBool(hideLogOut);
        _showJoin = !_safeBool(hideJoin);
        _showLogin = !_safeBool(hideLogin);

        _showLoginModalFlag = _safeBool(showLoginModal);
        _showJoinModalFlag = _safeBool(showJoinModal);

        if ("undefined" == typeof (contextData) || null == contextData) {
            _contextData = null;
        } else {
            _contextData = contextData;
        }

        _addWidgetEventListener();

        // We need an invisible iframe where we can run code inside the Wizards domain.
        // This iframe will have the internal widget and SSO widget available to JavaScript.
        var internalWidgetStartingUrl = _getInitInternalWidgetUrl(checkLogin, _requiredFieldSets);
        _buildUpInternalIframe(internalWidgetStartingUrl);

        // inject a stylesheet. Must be in initStart since it needs to be available even when the site is down.
        stylesheet = document.createElement("link");
        stylesheet.setAttribute("rel", "stylesheet");
        stylesheet.setAttribute("type", "text/css");
        stylesheet.setAttribute("href", _getSupportStylesheetUrl());

        document.head.appendChild(stylesheet);

        _debug("_initStart ending");
    };

    var _sendIsSiteUpPing = function() {
        var url = _getIsSiteUpUrl();
        
        var iframe = _createElement("iframe", { id: "isSiteUpPing", src: url, style: "display:none;" });

        document.getElementsByTagName("body")[0].appendChild(iframe);
    };

    var _initComplete = function (message) {
        if (_initCompleted) {
            _debug("_initComplete called (ignored; NOT the first time)");
            return;
        }
        _debug("_initComplete starting (first time)");
        _initCompleted = true;

        var doFallbackBehavior = false;

        _language = message.languageCode;
        _isMobile = message.isMobile;
        _setButtonText(message.buttonText);

        if (_showLoginModalFlag) {
            wotcAccountWidget.showLoginModal();
            _showLoginModalFlag = false;
        } else if (_showJoinModalFlag) {
            wotcAccountWidget.showJoinModal();
            _showJoinModalFlag = false;
        }

        if (message.checkLoginCase === "checkLoginCompleted") {
            var sessionToken = message.sessionToken;
            var rememberMeToken = message.rememberMeToken;
            doFallbackBehavior = _handleLoggedIn(sessionToken, rememberMeToken);
        } else if (message.checkLoginCase === "checkLoginCanceled") {
            doFallbackBehavior = true;
        } else if (message.checkLoginCase === "checkLoginSkipped") {
            doFallbackBehavior = true;
        } else {
            throw "Unrecognized message.checkLoginCase=" + message.case + " in _initComplete()";
        }

        if (doFallbackBehavior) {
            var controlsContainer = _isLoggedIn ?
                _getLoggedInControls(_showMyAccount, _showLogOut) :
                _getLoggedOutControls(_showJoin, _showLogin);

            if (controlsContainer) {
                _addClass(controlsContainer, "wotcWidget-defaultWidgetControls");
                
                if (!_container) {
                    _container = document.getElementsByTagName("body")[0];
                    _addClass(controlsContainer, "wotcWidget-defaultContainer");
                }
                controlsContainer.style.zIndex = _getTopZIndex();
                _container.appendChild(controlsContainer);
            }

            if (message.emailVerificationToken) {
                _showVerifyEmailModal(message.emailVerificationToken);
            }
        }

        _debug("_initComplete ending");

        // createEvent/initEvent are deprecated, but work cross-browser https://developer.mozilla.org/en-US/docs/Web/API/document.createEvent
        var event = document.createEvent("Event");
        event.initEvent("widgetInitialized", false, false);
        event.target = document;
        document.dispatchEvent(event);
    };

    var _setButtonText = function (buttonText) {
        if (buttonText) {
            if (buttonText.login && buttonText.login.length && buttonText.login.length > 0) {
                _buttonText.login = buttonText.login;
            }
            if (buttonText.logOut && buttonText.logOut.length && buttonText.logOut.length > 0) {
                _buttonText.logOut = buttonText.logOut;
            }
            if (buttonText.join && buttonText.join.length && buttonText.join.length > 0) {
                _buttonText.join = buttonText.join;
            }
            if (buttonText.myAccount && buttonText.myAccount.length && buttonText.myAccount.length > 0) {
                _buttonText.myAccount = buttonText.myAccount;
            }
            if (buttonText.confirmLogOutMessage && buttonText.confirmLogOutMessage.length && buttonText.confirmLogOutMessage.length > 0) {
                _buttonText.confirmLogOutMessage = buttonText.confirmLogOutMessage;
            }
        }
    };

    // public (privileged) functions
    this.showLoginModal = function (showCreateButton) {
        return _showLoginModal(showCreateButton);
    };

    this.showJoinModal = function () {
        return _showJoinModal();
    };

    this.showLanguageModal = function (reloadPageOnClose, openPreviousModalOnClose, previousModalUrl, contextData) {
        // Limitation here, we're not actually doing anything with the contextData object.
        return _showLanguageModal(reloadPageOnClose, openPreviousModalOnClose, previousModalUrl, contextData);
    };

    // This method will not return the language if you're invoking it too soon after
    // the initializing of the widget.
    this.getLanguage = function () {
        return _language;
    };

    this.getMyAccountsUrl = function () {
        return _getAccountDetailsPageUrl();
    };
    
    this.showAccountDetails = function (newTab) {
        return _showAccountDetails(newTab);
    };

    // Not used by AccountsWeb, but needed as exposed Widget API. Used by PWP, etc.
    this.hideModal = function () {
        return _hideModal();
    };

    this.logOut = function () {
        return _logOut();
    };

    // This method will set the context data.
    this.setContextData = function (contextData) {
        _contextData = contextData;
    };

    /*
        Each 3rd party app does its own session management.
        90% of the time this means that before the page is first served up,
        server-side code uses a token stored in a cookie or similar mechanism
        and makes server-side calls to SSO to determine the validity of the session.
        So as they third party page is rendering the call to init, it should already know whether it's logged in or not.
        That's what the loggedIn parameter is for.
        Note however that even if the app is logged out because their cookie is missing or timed out,
        init will check the wizards.com cookie and possibly call the onLoggedIn callback immediately.
    
        requiredFieldSets is a list of zero or more strings. It can be in any of the following formats:
        - a single string containing a single value
        - or, a single string containing multiple values delimited with +
        - or, an array of strings, or a single string listing multiple strings delimited with +, or null.
        - or, null
        - or, empty string
        In all of the above cases, white space is ignored (values are trimmed, etc.)
        Each value provided, if any, represents a set of fields that act together as a unit.
        Currently supported values are "name", "playlocation", and "screenname".
        The purpose of this parameter is so that Accounts can make sure that the user
        has all fields required by the app. If the user does have a WotC login,
        but doesn't have some fields required by the app, then they'll be treated as logged out,
        even if logged into the larger WotC ecosystem. This will require them to log in or join
        to use the app in question. The login and join processes will then require them to
        provide the required fields to access the app in question. Examples:
        - An app that requires no special fields should pass null or empty string ("").
        - An app that requires name and play location should pass:
          - ["name", "playlocation"]
          - or, new Array("name", "playlocation")
          - or, "name+playlocation"
        (Order does not matter.)
    
        container is optional. If not used then body is used.
        
        onLoggedIn is the callback function that we'll call when the user successfully logs in.
        See _handleLoggedIn() for what we do if undefined or depending on return value.
        
        onLoginCanceled is the callback function that we'll call when the user dismisses the modal without successfully logging in.
        See _handleLoginCanceled() for what we do if undefined or depending on return value.
        
        onLoggedOut is the callback function that we'll call when the user explicitly logs out.
        See _handleLoggedOut() for what we do if undefined.

        hideMyAccount, hideLogOut, hideJoin, hideLogin - pass true to suppress each individual button
    */

    this.init = function (isLoggedIn, requiredFieldSets, container, onLoggedIn, onLoginCanceled, onLoggedOut, onLanguageChanged, hideMyAccount, hideLogOut, hideJoin, hideLogin, showLoginModal, showJoinModal, contextData) {
        return _initStart(isLoggedIn, requiredFieldSets, container, onLoggedIn, onLoginCanceled, onLoggedOut, onLanguageChanged, hideMyAccount, hideLogOut, hideJoin, hideLogin, showLoginModal, showJoinModal, contextData);
    };

    this.isLoggedIn = function () {
        return _isLoggedIn;
    };

    this.requireAuthentication = function (requireAuthenticationCallback, defaultUsername) {
        return _requireAuthentication(requireAuthenticationCallback, defaultUsername);
    };

    // constants
    var _modalIframeName = "wotcAccountWidgetModalIframe";
    var _internalIframeName = "wotcAccountWidgetInternalIframe";
    var _touchScrollWrapperName = "touchScrollWrapper";

    var _accountDetailsPagePath = "/ExistingAccount/EditDetails";
    var _languageModalPath = "/Language/SelectLanguageWidget";

    var _isSiteUpPath = "/Default/IsSiteUp";
    var _maintenanceModalPath = "/Widget/Maintenance.html";
    var _supportStylesheetPath = "/Widget/WidgetSupport.css";
    
    var _loginModalPath = "/Widget/ShowLogin";
    var _joinModalPath = "/NewAccount/SignUp";
    var _initInternalWidgetPath = "/Widget/InitInternalWidget";
    var _initInternalWidgetWithCheckLoginPath = "/Widget/InitInternalWidgetWithCheckLogin";
    var _verifyEmailModalPath = "/NewAccount/VerifyEmail";

    var _rfsName = "name";
    var _rfsPlayLocation = "playlocation";
    var _rfsScreenName = "screenname";

    var _widgetHost = "accounts.wizards.com";
    var _accountsBaseUrl = (_widgetHost === ("@" + "Widget" + "Host" + "@"))
        ? _hackGetBaseUrlFromScriptTag()
        : "https://" + _widgetHost;
    
    // variables
    var _requiredFieldSets = null;

    var _onLoggedIn = null;
    var _onLoginCanceled = null;
    var _onLoggedOut = null;
    var _onLanguageChanged = null;

    var _showMyAccount = true;
    var _showLogOut = true;
    var _showJoin = true;
    var _showLogin = true;

    var _buttonText = {
        login: "Sign In",
        logOut: "Sign Out",
        join: "Join",
        myAccount: "My Account",
        confirmLogOutMessage: "You have made changes, if you log out now, the unsaved changes will be lost, cotinue?"
    };

    var _showLoginModalFlag = false;
    var _showJoinModalFlag = false;
    var _contextData = null;
    var _language = "en";
    var _isMobile = false;
    var _parentPageBody = null;

    var _initStarted = false;
    var _isSiteUp = false;
    var _initCompleted = false;

    var _isLoggedIn = false;

    var _inRequireAuthentication = false;
    var _requireAuthenticationCallback;

    var _container = null;
}
